@@ -1,9 +1,8 @@
 """Module with graph embedding models."""
 from .glove import GloVe
 from .skipgram import SkipGram
-from .binary_skipgram import BinarySkipGram
 from .cbow import CBOW
 
 __all__ = [
-    "GloVe", "SkipGram", "CBOW", "BinarySkipGram"
+    "GloVe", "SkipGram", "CBOW"
 ]
